package main

import (
	"database/sql"
	"net"

	migration "bitbucket.org/prmunty/util/dbmigration/db"
	"bitbucket.org/prmunty/util/ethernet"
	_ "github.com/mattn/go-sqlite3"
)

func main() {
	const version float64 = 9.0

	migration.Migrate(version, func(tx *sql.Tx) (err error) {
		return migration.DoUntilErr(tx,
			UpdateEquipmentTable,
			CreateModeOrdinalsTable,
			InsertModeOrdinals,
			UpdateFreezeTable,
			InsertFreezeData,
		)
	})

}

func UpdateEquipmentTable(tx *sql.Tx) error {
	rows, err := tx.Query(
		`SELECT ip FROM equipment`,
	);

	if err != nil {
		return err
	}

	for rows.Next() {
		var ip string

		err = rows.Scan(&ip)
		if err != nil {
			return err
		}

		netIp := net.ParseIP(ip)

		if ethernet.IsLocalIP(netIp) {
			if _, err = tx.Exec(
				`DELETE FROM equipment WHERE ip = ?`,
				ip,
			); err != nil {
				return err
			}
		}
	}

	return nil
}

func CreateModeOrdinalsTable(tx *sql.Tx) error {
	if _, err := tx.Exec(
		`DROP TABLE IF EXISTS mode_ordinals; CREATE TABLE mode_ordinals(mode_data Integer,text_index Integer,ordinal Integer)`,
	); err != nil {
		return err
	}

	return nil
}

func InsertModeOrdinals(tx *sql.Tx) error {
	userdefinedIds := map[int]int{
		0:1,
		1:2,
		2:5,
		3:3,
		4:4,
		5:6,
		6:7,
		7:38,
		8:39,
		9:40,
		10:41,
		11:42,
		12:43,
		13:44,
		14:45,
		15:46,
		16:47,
		21:7,
		22:8,
		23:9,
		24:10,
		25:11,
		26:12,
		27:13,
		28:14,
		29:15,
		30:16,
		31:17,
		32:18,
		33:19,
		34:20,
		35:21,
		36:22,
		37:23,
		38:24,
		39:25,
		40:26,
		41:27,
		42:28,
		43:29,
		44:30,
		45:31,
		46:32,
		47:33,
		48:34,
		49:35,
		50:36,
	}

	for key, value := range userdefinedIds {
		if _, err := tx.Exec(
			`INSERT INTO mode_ordinals(mode_data, text_index, ordinal) VALUES( ?, ?, ? )`,
			6790,
			key,
			value,
		); err != nil {
			return err
		}
	}

	return nil
}

func UpdateFreezeTable(tx *sql.Tx) error {
	if _, err := tx.Exec(
		`DROP TABLE IF EXISTS "freeze_data_evap"; CREATE TABLE "freeze_data_evap" ("freeze_id" Integer, "table_id" Integer, "zone" Integer,"selected" Boolean,"ordinal" Integer, PRIMARY KEY(freeze_id, zone));`,
	); err != nil {
		return err
	}

	return nil
}


func InsertFreezeData(tx *sql.Tx) error {
	freezeIds := map[int]int {
		359:38,
    503:2,
    504:3,
    505:4,
    245:5,
    2713:8,
    2714:9,
    2715:10,
    2716:11,
    2717:12,
    2718:13,
    2719:14,
    2720:15,
    2721:16,
    2722:17,
    2723:18,
    2724:19,
    2725:20,
    2726:21,
    2727:22,
    2728:23,
    2729:24,
    2730:25,
    2731:26,
    2732:27,
    2733:28,
    2734:29,
    2735:30,
    2736:31,
    2737:32,
    2738:33,
    2739:34,
    2740:35,
    2741:36,
    2742:37,
    502:1,
    2894:7,
    1872:39,
    1873:40,
    1874:41,
    1875:42,
    1876:43,
    1877:44,
    1878:45,
    1879:46,
    1880:47,
    1881:48,
    2460:6,
    1851:49,
    1852:50,
    2499:51,
    2500:52,
    2501:53,
    2502:54,
  }

	for zone := 0; zone < 30; zone++ {
		for key, value := range freezeIds {
			table_id := 0

			if _, err := tx.Exec(
				"INSERT INTO \"freeze_data_evap\" (\"freeze_id\",\"table_id\",\"zone\",\"selected\",\"ordinal\") VALUES(?, ?, ?, ?, ?);",
				key,
				table_id,
				zone,
				1,
				value,
			); err != nil {
				return err
			}
		}
	}

	return nil
}
