'use strict';
// DataBinder binds data to specific DOM elements.
// The DOM elements to which they are bound are determined by the json that
// represents the data that you want written to the DOM Elements.
// There are two ways to bind data to DOM Elements:
//      1. Write the data to DOM Elements that have a matching id in CamelCase format
//         as the name field in the JSON object. In other words the it removes
//         spaces from the name field looks for the DOM element with the
//         matching id.
//            Example:
//                The name value is "Suction Pressure". When the data is recieved
//                it is written the DOM Element with the id "SuctionPressure".
//      2. Write the data to DOM elements that have the bindingAttribute
//         variable. The value of the attribute must match the name field in the
//         JSON object that contains the data that you want to write to the DOM
//         Element. Note: Case of the value is not important
//  Note: Want to add a constructor that allows a user to
// determine what is assigned to what.
//
// Note on examples: We will assume that the bindingAttribute will be data-data-name
// Variables :
//       bindingAttribute: the attribute who's values will be checked against when searching for the name
//       receivers: a map that maps the bindingAttributes value to a list of
//           DOM elements
//          Example:
//               receivers["suction pressure"] = [<div data-data-name="suction pressure"></div>,
//                                               <span data-data-name="suction pressure"></span>]
//
//      times: an array of times that is added to when insert values is
//         called. This is used for benchmarking
function DataBinder(bindingAttribute) {
    this.times = [];
    this.bindingAttribute = bindingAttribute;
    this.recievers = {};


    /**** Function Definitions ****/

    // insertValue inserts assigns the dataValue to the value of the location
    // depending on the locations nodeName. It will either assign it to the value
    // attribute or write the data to the innerHTML
    function insertValue(location, dataValue){
        var tagType = location.nodeName;
        dataValue = dataValue.trim();
        // console.log("TAG TYPE: " + tagType);
        // console.log("DATA VALUE: " + dataValue);
        // console.log("THE ID: " + $(location).attr('id'));
        // console.log("----------------------------------------");
        switch (tagType) {
        case "INPUT":
            var units = $(location).val().replace(/[0-9.$-]/g, "").trim();
            $(location).attr('data-unit',units);
        case "TEXTAREA":
        case "SELECT":
            location.value = $("<div/>").html(dataValue).text();

            //console.log("\n\n***************-- " + location.name + " = " + dataValue + "\n\n");
            break;
        default:
            location.innerHTML = dataValue;
        }

        $(location).trigger("change");
    }


    // Inserts the values from an array of objects that contains the following fields
    //  1. name
    //  2. value
    // It will search for a matching 'name' in the recievers map and write the 'value'
    //  to the innerHTML of the DOM element for each element in the data list.
    function insertionViaCustomAttribute(dataList) {
        var time = Date.now();
        var index, locIndex, currPoint, name, locations, loc;
        for (index = 0; index < dataList.length; index += 1) {
            currPoint = dataList[index];
            //console.log(currPoint);
            name = currPoint.name.toLowerCase();
            locations = this.receivers[name];

            if (typeof locations === 'undefined') {
                this.elementError(name);

            } else {
                for (locIndex = 0; locIndex < locations.length; locIndex += 1) {
                    loc = locations[locIndex];
                    insertValue(loc, currPoint.value);
                }
            }
        }
        time = Date.now() - time;
        this.times.push(time);
    }

    // Inserts the values from an array of objects that contains the following fields
    //  1. name
    //  2. value
    // It will search for an id that matches the name with all the spaces removed in
    //  the DOM and write the 'value' to the innerHTML of the DOM element.
    function insertionViaId(dataList) {
        var time = Date.now();
        var regex = new RegExp(" ", 'g');
        var index, currPoint, location, element;
        for (index = 0; index < dataList.length; index += 1) {
            currPoint = dataList[index];
            location = currPoint.name.replace(regex, "");
            element = document.getElementById(location);
            if (typeof element === 'undefined') {
                this.elementError(location);
            } else {
                element.innerHTML = currPoint.value;
            }

        }

        time = Date.now() - time;
        this.times.push(time);
    }
    /**** End Function Definitions ****/

    switch (this.bindingAttribute) {
    case "class":
        throw "Cannot use an attributes class as a binding attribute.";
    case "id":
        this.insertValues = insertionViaId;
        break;
    default:
        // Initializes the recievers map so that we don't have to walk through the
        // DOM looking for everything matching the bindingAttribute
        var receiversArr, index, receiver, receiverName;
        receiversArr = document.querySelectorAll('[' + this.bindingAttribute + ']');
        this.receivers = {};
        for (index = 0; index < receiversArr.length; index += 1) {
            receiver = receiversArr[index];
            receiverName = (receiver.getAttribute(this.bindingAttribute)).toLowerCase();
            if (!this.receivers[receiverName]) {
                this.receivers[receiverName] = [];
            }

            this.receivers[receiverName].push(receiver);
        }

        this.insertValues = insertionViaCustomAttribute;
    }
}

// receivers is a map that maps the bindingAttribute's value to a list of  DOM elements
DataBinder.prototype.receivers = {};

// times is an array of times that is added to when insert values is
// called. This is used for benchmarking.
DataBinder.prototype.times = [];

//The attribute that identifys the element to which that data will be attached to.
DataBinder.prototype.bindingAttribute = "";

// insertValues inserts the values into the DOM Elements
DataBinder.prototype.insertValues = function () {};

// averageTime returns the averageTime that it took for insertValues to run.
DataBinder.prototype.averageTime = function () {
    var index, sum;
    sum = 0;
    for (index = 0; index < this.times.length; index += 1) {
        sum += this.times[index];

    }

    console.log(sum / this.times.length);
    return sum / this.times.length;

};

// Prints an error notifying the user that a matching element could
// not be found for the specified key
DataBinder.prototype.elementError = function (key) {
    console.error("Cannot find element where %s=\"%s\"",
                  this.bindingAttribute, key);

};







